MODULE GLOBAL
!!!!!!!!!!!!!!!!!!!!!!!!
!!   USE STATEMENTS   !!
!!!!!!!!!!!!!!!!!!!!!!!! 
USE PARAMETERS

IMPLICIT NONE

!! GRIDS
REAL(DP), DIMENSION(N_r)           	:: r_grid			! reserves
REAL(DP), DIMENSION(N_rp)          	:: rp_grid			! next period's reserves
REAL(DP), DIMENSION(N_rho)          :: rho_grid			! beliefs
REAL(DP), DIMENSION(N_rhop)         :: rhop_grid		! next period's beliefs
REAL(DP), DIMENSION(N_k)         	:: k_grid			! capital
REAL(DP), DIMENSION(N_phi)         	:: phi_grid			! liquidity shocks

!! POSTERIOR BELIEF MATRIX
REAL(DP), DIMENSION(N_rhop,N_phi,N_rho) :: rhop_pmf     ! matrix of posterior beliefs' pmf
REAL(DP), DIMENSION(N_phi,N_rho)    :: rhop_pmf_sum     ! conditional sum of posterior beliefs' pmf
REAL(DP), DIMENSION(N_rho)          :: rhop_pmf_min     ! min of conditional sum of posterior beliefs' pmf
REAL(DP), DIMENSION(N_rho)          :: rhop_pmf_max     ! min of conditional sum of posterior beliefs' pmf
REAL(DP), DIMENSION(N_rho)          :: rhop_pmf_avg

!! FEASIBILITY MAP
integer, DIMENSION(N_rho,N_r,N_k)   :: fsblmap		    ! feasible subcontract
integer, DIMENSION(N_rho,N_r)       :: fsblany		    ! indicates whether any subcontract is feasible
REAL(DP), DIMENSION(N_rho)          :: fsblmin		    ! indicates minimum reserves with feasible subcontract
REAL(DP), DIMENSION(N_rho,N_r,N_k)  :: fsblrsrv		    ! reserves for a feasible contract
REAL(DP), DIMENSION(N_rho,N_r,N_k)  :: fsblr_h		    ! normal time return for a feasible contract
REAL(DP), DIMENSION(N_rho,N_r,N_k)  :: fsblr_l		    ! sudden stop return for a feasible contract
REAL(DP), DIMENSION(N_rho,N_r,N_k)  :: fsblphi_lb	    ! lower bound - normal time for a feasible contract
REAL(DP), DIMENSION(N_rho,N_r,N_k)  :: fsblphi_ub	    ! upper bound - normal time for a feasible contract
REAL(DP), DIMENSION(N_rho,N_r,N_k)  :: fsbly_ss	        ! crisis time net output

!! VALUE FUNCTION & SCALAR POLICY FUNCTIONS
REAL(DP), DIMENSION (N_r,N_rho)     :: valfcn_opt		! value function
REAL(DP), DIMENSION (N_r,N_rho)     :: cap_opt			! optimal capital
integer, DIMENSION (N_r,N_rho)     	:: cap_idx_opt		! optimal capital index
REAL(DP), DIMENSION (N_r,N_rho)     :: rh_opt			! promised interest rate
REAL(DP), DIMENSION (N_r,N_rho)     :: res_opt			! optimal reserves
REAL(DP), DIMENSION (N_r,N_rho)     :: phi_s_opt		! sudden stops cutoff
REAL(DP), DIMENSION (N_r,N_rho)     :: phi_N_opt		! sudden stops cutoff
REAL(DP), DIMENSION (N_r,N_rho)     :: rl_opt			! sudden stop interest rate


!! INTERIM STATE-CONTIGENT POLICY FUNCTIONS
REAL(DP), DIMENSION (N_r,N_rho,N_phi,N_rhop)    :: liq_opt		! liquidation
integer, DIMENSION (N_r,N_rho,N_phi,N_rhop)     :: liq_idx_opt	! liquidation index
REAL(DP), DIMENSION (N_r,N_rho,N_phi,N_rhop)    :: rnext_opt    ! outgoing reserves saved
REAL(DP), DIMENSION (N_r,N_rho,N_phi,N_rhop)    :: cons_opt		! consumption
REAL(DP), DIMENSION (N_r,N_rho,N_phi,N_rhop)    :: policy_rp    ! saved reserves
REAL(DP), DIMENSION (N_r,N_rho,N_phi)           :: ss_opt       ! saved reserves
REAL(DP), DIMENSION (N_r,N_rho)                 :: phi_s_opt2   ! sudden stops cutoff
REAL(DP), DIMENSION (N_r,N_rho)                 :: phi_N_opt2	! sudden stops cutoff

!! SIMULATIONS
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: phi_simul        ! phi shock simulations
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: rold_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: k_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: ss_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: haircut_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: phir_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: phis_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: phin_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: rh_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: rl_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: rprime_simul
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: c_simul 
REAL(DP), DIMENSION (N_s,N_all_eme,t_end)       :: beliefs_simul
integer, DIMENSION (N_reg,N_eme)            	:: peers 			! matrix of peer-countries
integer, DIMENSION (N_all_eme)            	    :: regions 			! matrix of regions

!! CALIBRATION GRIDS
REAL(DP), DIMENSION (N_beta)        :: beta_grid		! calibration grid - beta
REAL(DP), DIMENSION (N_theta)       :: theta_grid	    ! calibration grid - theta
REAL(DP), DIMENSION (N_sigma_l)     :: sigma_l_grid	    ! calibration grid - sigma_l
REAL(DP), DIMENSION (N_sigma_h)     :: sigma_h_grid	    ! calibration grid - sigma_h

!! CALIBRATION GRIDS RESULTS

TYPE :: RESULTS_SIM
    
    ! simulation parametersto store
    REAL(DP)    :: beta_val
    REAL(DP)    :: theta_val
    REAL(DP)    :: sigma_l_val
    REAL(DP)    :: sigma_h_val
    
    ! simulation results
    REAL(DP)    :: r_ini_pre        ! reserves - pre period
    REAL(DP)    :: r_ini_trans      ! reserves - trans period
    REAL(DP)    :: r_ini_post       ! reserves - post period

    REAL(DP)    :: rho_pre          ! beliefs - pre period
    REAL(DP)    :: rho_trans        ! beliefs - trans period
    REAL(DP)    :: rho_post         ! beliefs - post period
    
    REAL(DP)    :: r_old_avg        ! incoming reserves - all periods

    REAL(DP)    :: ss_freq_pre      ! sudden stop freq - pre period
    REAL(DP)    :: ss_freq_trans    ! sudden stop freq - trans period
    REAL(DP)    :: ss_freq_post     ! sudden stop freq - post period    
    REAL(DP)    :: ss_freq_nontrans ! sudden stop freq - nontrans period
    
    REAL(DP)    :: r_h_pre          ! normal times yield - pre period
    REAL(DP)    :: r_h_trans        ! normal times yield - trans period
    REAL(DP)    :: r_h_post         ! normal times yield - post period
    
    REAL(DP)    :: haircut_all      ! average haircut - all periods
    REAL(DP)    :: haircut_cond     ! average haircut - when positive haircut 
        
    REAL(DP)    :: phi_s_pre        ! phi_s - pre period
    REAL(DP)    :: phi_s_trans      ! phi_s - trans period
    REAL(DP)    :: phi_s_post       ! phi_s - post period
    
    REAL(DP)    :: phi_n_pre        ! phi_n - pre period
    REAL(DP)    :: phi_n_trans      ! phi_n - trans period
    REAL(DP)    :: phi_n_post       ! phi_n - post period
    
    REAL(DP)    :: ss_count_pre     ! sudden stops - pre period
    REAL(DP)    :: ss_count_trans   ! sudden stops - trans period
    REAL(DP)    :: ss_count_post    ! sudden stops - post period
    
    REAL(DP)    :: rold_erg_min     ! minimum rold in ergodic distribution
    REAL(DP)    :: rold_erg_max     ! maximum rold in ergodic distribution
    
    REAL(DP), DIMENSION (N_hist)    :: ss_hist_pre      ! histogram of sudden stops - pre period
    REAL(DP), DIMENSION (N_hist)    :: ss_hist_trans    ! histogram of sudden stops - trans period
    REAL(DP), DIMENSION (N_hist)    :: ss_hist_post     ! histogram of sudden stops - post period
    
END TYPE RESULTS_SIM

TYPE (RESULTS_SIM),                 &
    DIMENSION ( N_beta,             &
                N_theta,            &
                N_sigma_l,          &
                N_sigma_h )         :: results_grid  ! results

END MODULE GLOBAL