MODULE PARAMETERS

IMPLICIT NONE

INTEGER, PARAMETER  ::  DP          = SELECTED_REAL_KIND(P=13)

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! BASELINE CALIBRATION !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

REAL(DP)            ::  beta        = 0.931246599920233_dp,     &       ! government discount factor
						theta 		= 0.814995704309168_dp,     &    	! renegotiation bargaining parameter
                        sigma_l 	= 0.059892498083833_dp,     &    	! low rollover risk
						sigma_h 	= 0.174760401224615_dp              ! high rollover risk

!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! CONSTANT PARAMETERS !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

REAL(DP), PARAMETER ::  r			= 1.01_dp, 		& 		! world interest rate
						alpha 		= 1.0_dp,  		&    	! curvature on production function (baseline: linear)
                        lambda 		= 0.60_dp,     	&  		! divestment return (baseline: 0.60)
						A 			= 1.20_dp,		&	    ! productivity (baseline: 1.20)
						debt 		= 1.0_dp,       &       ! debt level (normalized to 1)
                        onedp		= 1.0_dp,       &       ! double precision - one
                        zerodp		= 0.0_dp,       &       ! double precision - zero
                        rho_init    = 0.95_dp,      &       ! initial prior
                        rold_init   = 0.2_dp                ! initial saved reserves

INTEGER, PARAMETER ::   N_r 		= 40,     		&    	! grid points for incoming reserves
						N_rp 		= 20,     		&    	! grid points for saved reserves
						N_rho 		= 5,     		&    	! grid points for prior
						N_rhop 		= 10,    		&    	! grid points for posterior
						N_k 		= 60,     		&    	! grid points for investment
						N_phi 		= 150,        	&		! grid points for phi shocks
                        N_eme 		= 23,           &		! number of economies for learning - within regions
                        N_reg       = 01,           &		! number of regions
                        N_all_eme   = N_eme*N_reg,  &		! number of economies in the world
                        N_s         = 1000,         &       ! number of simulations
                        T_erg       = 100,          &       ! number of pre-simulation periods to ensure ergodicity
                        T_end       = 100 + t_erg,  &       ! number of periods per simulation
                        T_start     = 010 + t_erg,  &       ! start time
                        T_change    = 050 + t_erg,  &       ! switch to high sigma
                        q_pre       = 20,           &       ! number of quarters in pre
                        q_post      = 20,           &       ! number of quarters in post
                        q_trans     = 20                    ! number of quarters in transition

REAL(DP), PARAMETER ::  rmax        = 0.650_dp,         &   ! upper bound on saved reserves grid
                        rmin        = 0.000_dp,         &   ! lower bound on saved reserves grid
						kmax        = 1.000_dp,         &   ! upper bound on capital grid
                        kmin        = 1.00d-12,         &   ! lower bound on capital grid
						rhmax       = r + 0.030_dp,     &   ! upper bound on interest rate grid
                        rhmin       = r + 0.000_dp,     &   ! lower bound on interest rate grid
                        phimin      = 1.0d-7,           &   ! lower bound on phi grid
                        phimax      = 1.0_dp - phimin,  &   ! upper bound on phi grid
                        rhomin      = 1.0d-7,           &   ! lower bound on rho grid
                        rhomax      = 1.0_dp - rhomin,  &   ! upper bound on rho grid
                        rhopmin     = 1.0d-7,           &   ! lower bound on rhop grid
                        rhopmax     = 1.0_dp - rhopmin, &   ! upper bound on rhop grid
                        lmin        = 0.0d+0,           &   ! lower bound on l grid
                        lmax        = 1.0d+0                ! upper bound on l grid


INTEGER, PARAMETER  ::  N_hist      = 19                    ! number of bins for sudden stop counts

REAL(DP), PARAMETER ::  grdpwr      = 1.80_dp,          &   ! curvature > 1 for more points near 0
                        onepwr      = 1.00_dp               ! curvature for even spacing

INTEGER, PARAMETER  ::  izero       = 1                     ! index of zero reserves on incoming grid

REAL(DP), PARAMETER ::  convcrit   	= 1.0d-2,           &   ! value function convergence criterion
                        polcrit   	= 1.0d-10,          &   ! policy function convergence criterion
                        infty       = 1.0d12,           &   ! large positive value
						eps   		= 1.0d-12,          &   ! epsilon
                        weight      = 0.20_dp               ! speed of convergence parameter

INTEGER, PARAMETER  ::  vf_it_max   = 100                   ! maximum number of value function iterations

INTEGER, PARAMETER  ::  N_ints      = 50                    ! number of subintervals - bisection by subintervals

INTEGER, PARAMETER  ::  cdf_force   = 1                     ! forces the cdf of rhop to be 1 always

INTEGER, PARAMETER ::   trace_track	= 0,                &   ! determines whether program high level parts tracked
                        trace_code	= 0,                &   ! determines whether output traced in detail
                        save_vars	= 1,                &   ! determines whether output saved
                        step_l      = 2,                &   ! step for printing tables
                        step_h      = 9                     ! step for printing tables


!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!!!! CALIBRATION PARAMETERS !!!!
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!

REAL(DP)            ::  betamin     = 0.931246599920233_dp,     &       ! government discount factor - min
                        betamax     = 0.931246599920233_dp,     &       ! government discount factor - max
						thetamin	= 0.814995704309168_dp,     &    	! renegotiation bargaining parameter - min
                        thetamax	= 0.814995704309168_dp,     &    	! renegotiation bargaining parameter - max
                        sigma_lmin 	= 0.059892498083833_dp,     &    	! low rollover risk - min
                        sigma_lmax 	= 0.059892498083833_dp,     &    	! low rollover risk - max
                        sigma_hmin 	= 0.174760401224615_dp,     &    	! high rollover risk - min
						sigma_hmax 	= 0.174760401224615_dp              ! high rollover risk - max

INTEGER, PARAMETER  ::  N_beta      = 1,            &       ! sample points for calibration - beta
                        N_theta     = 1,            &       ! sample points for calibration - theta
                        N_sigma_l   = 1,            &       ! sample points for calibration - sigma_l
                        N_sigma_h   = 1                     ! sample points for calibration - sigma_h

END MODULE PARAMETERS
